From 4a4f2e2377c1f5d72fef28b47657e7f5f70417ef Mon Sep 17 00:00:00 2001
From: Rushabh Lathia <rushabh.lathia@enterprisedb.com>
Date: Fri, 8 Dec 2017 11:58:11 +0530
Subject: [PATCH 2/2] Adjust planner code to consider
 parallel_leader_participation.

---
 src/backend/access/nbtree/nbtsort.c  | 2 +-
 src/backend/optimizer/plan/planner.c | 7 +++++--
 src/include/optimizer/planner.h      | 3 ++-
 3 files changed, 8 insertions(+), 4 deletions(-)

diff --git a/src/backend/access/nbtree/nbtsort.c b/src/backend/access/nbtree/nbtsort.c
index 2b58621..b2661a2 100644
--- a/src/backend/access/nbtree/nbtsort.c
+++ b/src/backend/access/nbtree/nbtsort.c
@@ -398,7 +398,7 @@ _bt_heapscan(Relation heap, Relation index, BTBuildState *buildstate,
 		int			request;
 
 		/* Determine optimal number of worker processes, and attempt launch */
-		request = plan_create_index_workers(table, index);
+		request = plan_create_index_workers(table, index, leaderasworker);
 
 		if (request > 0)
 		{
diff --git a/src/backend/optimizer/plan/planner.c b/src/backend/optimizer/plan/planner.c
index 8ff32bd..eff1573 100644
--- a/src/backend/optimizer/plan/planner.c
+++ b/src/backend/optimizer/plan/planner.c
@@ -6210,6 +6210,8 @@ get_partitioned_child_rels_for_join(PlannerInfo *root, Relids join_relids)
  *
  * tableOid is the table that index is to be built on.  indexOid is the OID
  * of a index to be created or reindexed (which must be a btree index).
+ * leaderasworker indicates whether leader going to participate as worker or
+ * not.
  *
  * Return value is the number of parallel workers to use, which can often be
  * treated as a suggestion.  However, a return value of 0 may be due to it
@@ -6220,7 +6222,7 @@ get_partitioned_child_rels_for_join(PlannerInfo *root, Relids join_relids)
  * index.
  */
 int
-plan_create_index_workers(Oid tableOid, Oid indexOid)
+plan_create_index_workers(Oid tableOid, Oid indexOid, bool leaderasworker)
 {
 	PlannerInfo *root;
 	Query	   *query;
@@ -6236,6 +6238,7 @@ plan_create_index_workers(Oid tableOid, Oid indexOid)
 	BlockNumber heap_blocks;
 	double		reltuples;
 	double		allvisfrac;
+	int			leader_as_worker = leaderasworker ? 1 : 0;
 
 	/*
 	 * Fast-path:  Return immediately when parallelism disabled.  Note that we
@@ -6326,7 +6329,7 @@ plan_create_index_workers(Oid tableOid, Oid indexOid)
 	sort_mem_blocks = (maintenance_work_mem * 1024L) / BLCKSZ;
 	min_sort_mem_blocks = (32768L * 1024L) / BLCKSZ;
 	while (parallel_workers > min_parallel_workers &&
-		   sort_mem_blocks / (parallel_workers + 1) < min_sort_mem_blocks)
+		   sort_mem_blocks / (parallel_workers + leader_as_worker) < min_sort_mem_blocks)
 		parallel_workers--;
 
 done:
diff --git a/src/include/optimizer/planner.h b/src/include/optimizer/planner.h
index 1603b67..5c89949 100644
--- a/src/include/optimizer/planner.h
+++ b/src/include/optimizer/planner.h
@@ -61,6 +61,7 @@ extern List *get_partitioned_child_rels(PlannerInfo *root, Index rti);
 extern List *get_partitioned_child_rels_for_join(PlannerInfo *root,
 									Relids join_relids);
 
-extern int	plan_create_index_workers(Oid tableOid, Oid indexOid);
+extern int plan_create_index_workers(Oid tableOid, Oid indexOid,
+						  bool leaderasworker);
 
 #endif							/* PLANNER_H */
-- 
1.8.3.1

